"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Collections = require("typescript-collections");
var uml = require("../uml/index");
var formatter_1 = require("./formatter");
/**
 * yUML Formatter
 *
 * https://yuml.me/diagram/scruffy/class/samples
 *
 * @export
 * @class Formatter
 * @extends {AbstractFormatter}
 */
var Formatter = (function (_super) {
    __extends(Formatter, _super);
    function Formatter(options) {
        var _this = _super.call(this, options) || this;
        _this._outputtedNodes = new Collections.Set();
        return _this;
    }
    Formatter.prototype.generateClassDiagram = function (umlCodeModel) {
        var _this = this;
        var yuml = "// {type:class}\n";
        this._outputtedNodes.clear();
        umlCodeModel.generalizations.forEach(function (value) {
            yuml += _this._formatNode(umlCodeModel.nodes.getValue(value.toName));
            yuml += "^" + _this._formatNode(umlCodeModel.nodes.getValue(value.fromName)) + "\n";
            _this._outputtedNodes.add(value.fromName);
            _this._outputtedNodes.add(value.toName);
        });
        umlCodeModel.associations.forEach(function (link) {
            var reverse = link.reverse();
            var formattedAssociation = null;
            if (umlCodeModel.associations.contains(reverse)) {
                if (link.toString().localeCompare(reverse.toString()) < 0) {
                    formattedAssociation = _this._formatNode(umlCodeModel.nodes.getValue(link.fromName));
                    formattedAssociation += "-" + _this._formatNode(umlCodeModel.nodes.getValue(link.toName)) + "\n";
                }
            }
            else {
                formattedAssociation = _this._formatNode(umlCodeModel.nodes.getValue(link.fromName));
                formattedAssociation += "->" + _this._formatNode(umlCodeModel.nodes.getValue(link.toName)) + "\n";
            }
            if (formattedAssociation) {
                yuml += formattedAssociation;
                _this._outputtedNodes.add(link.fromName);
                _this._outputtedNodes.add(link.toName);
            }
        });
        umlCodeModel.nodes.forEach(function (key, value) {
            if (!_this._outputtedNodes.contains(key)) {
                yuml += _this._formatNode(value) + "\n";
            }
        });
        return yuml;
    };
    Formatter.prototype._formatNode = function (node) {
        if (node instanceof uml.Class) {
            var properties = this._formatProperties(node);
            switch (node.stereotype) {
                case 2 /* Interface */:
                    return "[<<" + node.identifier + ">>|" + properties + "]";
                default:
                    return "[" + node.identifier + "|" + properties + "]";
            }
        }
    };
    Formatter.prototype._formatProperties = function (node) {
        var _this = this;
        var variables = node.variables.values().map(function (variable) {
            return _this._formatVariable(variable);
        }).join(";");
        var methods = node.methods.values().map(function (method) {
            return _this._formatMethod(method);
        }).join(";");
        return this._replaceSpecialCharacters(variables + "|" + methods);
    };
    Formatter.prototype._formatVariable = function (variable) {
        var accessibility = this._formatAccessibility(variable.accessibility);
        return "" + accessibility + variable.identifier + ":" + variable.type.text;
    };
    Formatter.prototype._formatMethod = function (method) {
        var accessibility = this._formatAccessibility(method.accessibility);
        var returns = "";
        if (method.returnType) {
            returns = ":" + method.returnType.text;
        }
        var parameters = method.parameters.map(function (p) {
            var type = "";
            var initializer = "";
            var questionMark = "";
            if (p.type) {
                type = ":" + p.type.text;
            }
            if (p.defaultInitializer) {
                initializer = "=" + p.defaultInitializer;
            }
            else if (p.optional) {
                questionMark = "?";
            }
            return "" + p.identifier + questionMark + type + initializer;
        });
        return "" + accessibility + method.identifier + "(" + parameters.join(",") + ")" + returns;
    };
    Formatter.prototype._formatAccessibility = function (accessibility) {
        var result = "";
        switch (accessibility) {
            default:
            case 0 /* Public */:
                result = "+";
                break;
            case 1 /* Protected */:
                result = "#";
                break;
            case 2 /* Private */:
                result = "-";
                break;
        }
        return result;
    };
    Formatter.prototype._replaceSpecialCharacters = function (value) {
        return value
            .replace(/\[/g, "［")
            .replace(/\]/g, "］");
    };
    return Formatter;
}(formatter_1.AbstractFormatter));
exports.Formatter = Formatter;
